@@ -34,10 +34,16 @@
  */
 
 /*
- * User to drop privileges to by default.
+ * User to drop privileges to by default.  This user needs to be allowed to
+ * create outbound TCP connections, and in some configurations, perform DNS
+ * resolution.
+ *
  * Packagers may want to use a specific service user account instead of
  * overloading nobody with yet another use case.  Using nobody for source
- * builds makes sense because chances are high that it exists.
+ * builds makes sense because chances are high that it exists.  Good practice
+ * is to create a dedicated user for sslsplit.
+ *
+ * Make sure to also patch the manual page if you patch this.
  */
 #define DFLT_DROPUSER "nobody"
 
@@ -63,6 +69,17 @@
  */
 #define DFLT_CURVE "prime256v1"
 
+/*
+ * Default leaf key RSA keysize in bits.
+ *
+ * While browsers still generally accept it, use a leaf key size of 1024 bit
+ * for leaf keys.  When browsers start to sunset 1024 bit RSA in leaf keys, we
+ * will need to make this value bigger, and/or configurable.
+ * Until then, users who want a different size can always use their own
+ * pre-generated leaf key instead of generating one automatically.
+ */
+#define DFLT_LEAFKEY_RSABITS 1024
+
 #endif /* !DEFAULTS_H */
 
 /* vim: set noet ft=c: */
