@@ -53,10 +53,13 @@
 #include <getopt.h>
 #endif /* !__BSD__ */
 
-#include <event2/event.h>
-
 #include <openssl/ssl.h>
 #include <openssl/x509.h>
+#include <event2/event.h>
+#ifndef WITHOUT_MIRROR
+#include <libnet.h>
+#include <pcap.h>
+#endif /* !WITHOUT_MIRROR */
 
 #if __APPLE__
 #undef daemon
@@ -114,6 +117,22 @@ main_version(void)
 	ssl_openssl_version();
 	fprintf(stderr, "compiled against libevent %s\n", LIBEVENT_VERSION);
 	fprintf(stderr, "rtlinked against libevent %s\n", event_get_version());
+#ifndef WITHOUT_MIRROR
+	fprintf(stderr, "compiled against libnet %s\n", LIBNET_VERSION);
+#ifndef __OpenBSD__
+	const char *lnv = libnet_version();
+	if (!strncmp(lnv, "libnet version ", 15))
+		lnv += 15;
+	fprintf(stderr, "rtlinked against libnet %s\n", lnv);
+#else /* __OpenBSD__ */
+	fprintf(stderr, "rtlinked against libnet n/a\n");
+#endif /* __OpenBSD__ */
+	fprintf(stderr, "compiled against libpcap n/a\n");
+	const char *lpv = pcap_lib_version();
+	if (!strncmp(lpv, "libpcap version ", 16))
+		lpv += 16;
+	fprintf(stderr, "rtlinked against libpcap %s\n", lpv);
+#endif /* !WITHOUT_MIRROR */
 	fprintf(stderr, "%d CPU cores detected\n", sys_get_cpu_cores());
 }
 
@@ -124,8 +143,10 @@ static void
 main_usage(void)
 {
 	const char *dflt, *warn;
-	const char *usagefmt =
-"Usage: %s [options...] [proxyspecs...]\n"
+	const char *usagefmt1 =
+"Usage: %s [-D] [-f conffile] [-o opt=val] [options...] [proxyspecs...]\n"
+"  -f conffile use conffile to load configuration from\n"
+"  -o opt=val  override conffile option opt with value val\n"
 "  -c pemfile  use CA cert (and key) from pemfile to sign forged certs\n"
 "  -k pemfile  use CA key (and cert) from pemfile to sign forged certs\n"
 "  -C pemfile  use CA chain from pemfile (intermediate and root CA certs)\n"
@@ -161,6 +182,12 @@ main_usage(void)
 "  -r proto    only support one of " SSL_PROTO_SUPPORT_S "(default: all)\n"
 "  -R proto    disable one of " SSL_PROTO_SUPPORT_S "(default: none)\n"
 "  -s ciphers  use the given OpenSSL cipher suite spec (default: " DFLT_CIPHERS ")\n"
+#ifndef OPENSSL_NO_ENGINE
+"  -x engine   load OpenSSL engine with the given identifier\n"
+#define OPT_x "x:"
+#else /* OPENSSL_NO_ENGINE */
+#define OPT_x 
+#endif /* OPENSSL_NO_ENGINE */
 "  -e engine   specify default NAT engine to use (default: %s)\n"
 "  -E          list available NAT engines and exit\n"
 "  -u user     drop privileges to user (default if run as root: " DFLT_DROPUSER ")\n"
@@ -187,17 +214,34 @@ main_usage(void)
 "              %%%% - literal '%%'\n"
 #ifdef HAVE_LOCAL_PROCINFO
 "      e.g.    \"/var/log/sslsplit/%%X/%%u-%%s-%%d-%%T.log\"\n"
+#else /* !HAVE_LOCAL_PROCINFO */
+"      e.g.    \"/var/log/sslsplit/%%T-%%s-%%d.log\"\n"
+#endif /* HAVE_LOCAL_PROCINFO */
+"  -X pcapfile pcap log: packets to pcapfile (excludes -Y/-y)\n"
+"  -Y pcapdir  pcap log: packets to separate files in dir (excludes -X/-y)\n"
+"  -y pathspec pcap log: packets to sep files with %% subst (excl. -X/-Y):\n"
+"              see option -F for pathspec format\n"
+#ifndef WITHOUT_MIRROR
+"  -I if       mirror packets to interface\n"
+"  -T addr     mirror packets to target address (used with -I)\n"
+#define OPT_I "I:"
+#define OPT_T "T:"
+#else /* WITHOUT_MIRROR */
+#define OPT_I 
+#define OPT_T 
+#endif /* WITHOUT_MIRROR */
+"  -M logfile  log master keys to logfile in SSLKEYLOGFILE format\n"
+#ifdef HAVE_LOCAL_PROCINFO
 "  -i          look up local process owning each connection for logging\n"
 #define OPT_i "i"
 #else /* !HAVE_LOCAL_PROCINFO */
-"      e.g.    \"/var/log/sslsplit/%%T-%%s-%%d.log\"\n"
 #define OPT_i 
 #endif /* HAVE_LOCAL_PROCINFO */
-"  -M logfile  log master keys to logfile in SSLKEYLOGFILE format\n"
 "  -d          daemon mode: run in background, log error messages to syslog\n"
 "  -D          debug mode: run in foreground, log debug messages on stderr\n"
 "  -V          print version information and exit\n"
-"  -h          print usage information and exit\n"
+"  -h          print usage information and exit\n";
+	const char *usagefmt2 =
 "  proxyspec = type listenaddr+port [natengine|targetaddr+port|\"sni\"+port]\n"
 "      e.g.    http 0.0.0.0 8080 www.roe.ch 80  # http/4; static hostname dst\n"
 "              https ::1 8443 2001:db8::1 443   # https/6; static address dst\n"
@@ -217,7 +261,8 @@ main_usage(void)
 		warn = "";
 	}
 
-	fprintf(stderr, usagefmt, build_pkgname, dflt, build_pkgname, warn);
+	fprintf(stderr, usagefmt1, build_pkgname, dflt);
+	fprintf(stderr, usagefmt2, build_pkgname, warn);
 }
 
 /*
@@ -274,18 +319,6 @@ main_loadtgcrt(const char *filename, void *arg)
 	return 0;
 }
 
-/*
- * Handle out of memory conditions in early stages of main().
- * Print error message and exit with failure status code.
- * Does not return.
- */
-void NORET
-oom_die(const char *argv0)
-{
-	fprintf(stderr, "%s: out of memory\n", argv0);
-	exit(EXIT_FAILURE);
-}
-
 /*
  * Main entry point.
  */
@@ -309,220 +342,91 @@ main(int argc, char *argv[])
 		natengine = NULL;
 	}
 
-	while ((ch = getopt(argc, argv, OPT_g OPT_G OPT_Z OPT_i "k:c:C:K:t:OPa:"
-	                    "b:s:r:R:e:Eu:m:j:p:l:L:S:F:M:dDVhW:w:q:")) != -1) {
+	while ((ch = getopt(argc, argv,
+	                    OPT_g OPT_G OPT_Z OPT_i OPT_x OPT_T OPT_I
+	                    "k:c:C:K:t:OPa:b:s:r:R:e:Eu:m:j:p:l:L:S:F:M:"
+	                    "dDVhW:w:q:f:o:X:Y:y:")) != -1) {
 		switch (ch) {
-			case 'c':
-				if (opts->cacrt)
-					X509_free(opts->cacrt);
-				opts->cacrt = ssl_x509_load(optarg);
-				if (!opts->cacrt) {
-					fprintf(stderr, "%s: error loading CA "
-					                "cert from '%s':\n",
-					                argv0, optarg);
-					if (errno) {
-						fprintf(stderr, "%s\n",
-						        strerror(errno));
-					} else {
-						ERR_print_errors_fp(stderr);
-					}
+			case 'f':
+				if (opts->conffile)
+					free(opts->conffile);
+				opts->conffile = strdup(optarg);
+				if (!opts->conffile)
+					oom_die(argv0);
+				if (load_conffile(opts, argv0, &natengine) == -1) {
 					exit(EXIT_FAILURE);
 				}
-				ssl_x509_refcount_inc(opts->cacrt);
-				sk_X509_insert(opts->chain, opts->cacrt, 0);
-				if (!opts->cakey) {
-					opts->cakey = ssl_key_load(optarg);
+#ifdef DEBUG_OPTS
+				fprintf(stderr, "Conf file: %s\n", opts->conffile);
+#endif /* DEBUG_OPTS */
+				break;
+			case 'o':
+				if (opts_set_option(opts, argv0, optarg, &natengine) == -1) {
+					exit(EXIT_FAILURE);
 				}
-#ifndef OPENSSL_NO_DH
-				if (!opts->dh) {
-					opts->dh = ssl_dh_load(optarg);
-				}
-#endif /* !OPENSSL_NO_DH */
+				break;
+			case 'c':
+				opts_set_cacrt(opts, argv0, optarg);
 				break;
 			case 'k':
-				if (opts->cakey)
-					EVP_PKEY_free(opts->cakey);
-				opts->cakey = ssl_key_load(optarg);
-				if (!opts->cakey) {
-					fprintf(stderr, "%s: error loading CA "
-					                "key from '%s':\n",
-					                argv0, optarg);
-					if (errno) {
-						fprintf(stderr, "%s\n",
-						        strerror(errno));
-					} else {
-						ERR_print_errors_fp(stderr);
-					}
-					exit(EXIT_FAILURE);
-				}
-				if (!opts->cacrt) {
-					opts->cacrt = ssl_x509_load(optarg);
-					if (opts->cacrt) {
-						ssl_x509_refcount_inc(
-						               opts->cacrt);
-						sk_X509_insert(opts->chain,
-						               opts->cacrt, 0);
-					}
-				}
-#ifndef OPENSSL_NO_DH
-				if (!opts->dh) {
-					opts->dh = ssl_dh_load(optarg);
-				}
-#endif /* !OPENSSL_NO_DH */
+				opts_set_cakey(opts, argv0, optarg);
 				break;
 			case 'C':
-				if (ssl_x509chain_load(NULL, &opts->chain,
-				                       optarg) == -1) {
-					fprintf(stderr, "%s: error loading "
-					                "chain from '%s':\n",
-					                argv0, optarg);
-					if (errno) {
-						fprintf(stderr, "%s\n",
-						        strerror(errno));
-					} else {
-						ERR_print_errors_fp(stderr);
-					}
-					exit(EXIT_FAILURE);
-				}
+				opts_set_chain(opts, argv0, optarg);
 				break;
 			case 'K':
-				if (opts->key)
-					EVP_PKEY_free(opts->key);
-				opts->key = ssl_key_load(optarg);
-				if (!opts->key) {
-					fprintf(stderr, "%s: error loading lea"
-					                "f key from '%s':\n",
-					                argv0, optarg);
-					if (errno) {
-						fprintf(stderr, "%s\n",
-						        strerror(errno));
-					} else {
-						ERR_print_errors_fp(stderr);
-					}
-					exit(EXIT_FAILURE);
-				}
-#ifndef OPENSSL_NO_DH
-				if (!opts->dh) {
-					opts->dh = ssl_dh_load(optarg);
-				}
-#endif /* !OPENSSL_NO_DH */
+				opts_set_key(opts, argv0, optarg);
 				break;
 			case 't':
-				if (!sys_isdir(optarg)) {
-					fprintf(stderr, "%s: '%s' is not a "
-					                "directory\n",
-					                argv0, optarg);
-					exit(EXIT_FAILURE);
-				}
-				if (opts->tgcrtdir)
-					free(opts->tgcrtdir);
-				opts->tgcrtdir = strdup(optarg);
-				if (!opts->tgcrtdir)
-					oom_die(argv0);
+				opts_set_tgcrtdir(opts, argv0, optarg);
 				break;
 			case 'q':
-				if (opts->crlurl)
-					free(opts->crlurl);
-				opts->crlurl = strdup(optarg);
+				opts_set_crl(opts, optarg);
 				break;
 			case 'O':
-				opts->deny_ocsp = 1;
+				opts_set_deny_ocsp(opts);
 				break;
 			case 'P':
-				opts->passthrough = 1;
+				opts_set_passthrough(opts);
 				break;
 			case 'a':
-				if (opts->clientcrt)
-					X509_free(opts->clientcrt);
-				opts->clientcrt = ssl_x509_load(optarg);
-				if (!opts->clientcrt) {
-					fprintf(stderr, "%s: error loading cli"
-					                "ent cert from '%s':\n",
-					                argv0, optarg);
-					if (errno) {
-						fprintf(stderr, "%s\n",
-						        strerror(errno));
-					} else {
-						ERR_print_errors_fp(stderr);
-					}
-					exit(EXIT_FAILURE);
-				}
+				opts_set_clientcrt(opts, argv0, optarg);
 				break;
 			case 'b':
-				if (opts->clientkey)
-					EVP_PKEY_free(opts->clientkey);
-				opts->clientkey = ssl_key_load(optarg);
-				if (!opts->clientkey) {
-					fprintf(stderr, "%s: error loading cli"
-					                "ent key from '%s':\n",
-					                argv0, optarg);
-					if (errno) {
-						fprintf(stderr, "%s\n",
-						        strerror(errno));
-					} else {
-						ERR_print_errors_fp(stderr);
-					}
-					exit(EXIT_FAILURE);
-				}
+				opts_set_clientkey(opts, argv0, optarg);
 				break;
 #ifndef OPENSSL_NO_DH
 			case 'g':
-				if (opts->dh)
-					DH_free(opts->dh);
-				opts->dh = ssl_dh_load(optarg);
-				if (!opts->dh) {
-					fprintf(stderr, "%s: error loading DH "
-					                "params from '%s':\n",
-					                argv0, optarg);
-					if (errno) {
-						fprintf(stderr, "%s\n",
-						        strerror(errno));
-					} else {
-						ERR_print_errors_fp(stderr);
-					}
-					exit(EXIT_FAILURE);
-				}
+				opts_set_dh(opts, argv0, optarg);
 				break;
 #endif /* !OPENSSL_NO_DH */
 #ifndef OPENSSL_NO_ECDH
 			case 'G':
-			{
-				EC_KEY *ec;
-				if (opts->ecdhcurve)
-					free(opts->ecdhcurve);
-				if (!(ec = ssl_ec_by_name(optarg))) {
-					fprintf(stderr, "%s: unknown curve "
-					                "'%s'\n",
-					                argv0, optarg);
-					exit(EXIT_FAILURE);
-				}
-				EC_KEY_free(ec);
-				opts->ecdhcurve = strdup(optarg);
-				if (!opts->ecdhcurve)
-					oom_die(argv0);
+				opts_set_ecdhcurve(opts, argv0, optarg);
 				break;
-			}
 #endif /* !OPENSSL_NO_ECDH */
 #ifdef SSL_OP_NO_COMPRESSION
 			case 'Z':
-				opts->sslcomp = 0;
+				opts_unset_sslcomp(opts);
 				break;
 #endif /* SSL_OP_NO_COMPRESSION */
 			case 's':
-				if (opts->ciphers)
-					free(opts->ciphers);
-				opts->ciphers = strdup(optarg);
-				if (!opts->ciphers)
-					oom_die(argv0);
+				opts_set_ciphers(opts, argv0, optarg);
 				break;
 			case 'r':
-				opts_proto_force(opts, optarg, argv0);
+				opts_force_proto(opts, argv0, optarg);
 				break;
 			case 'R':
-				opts_proto_disable(opts, optarg, argv0);
+				opts_disable_proto(opts, argv0, optarg);
 				break;
+#ifndef OPENSSL_NO_ENGINE
+			case 'x':
+				opts_set_openssl_engine(opts, argv0, optarg);
+				break;
+#endif /* !OPENSSL_NO_ENGINE */
 			case 'e':
-				free(natengine);
+				if (natengine)
+					free(natengine);
 				natengine = strdup(optarg);
 				if (!natengine)
 					oom_die(argv0);
@@ -532,199 +436,65 @@ main(int argc, char *argv[])
 				exit(EXIT_SUCCESS);
 				break;
 			case 'u':
-				if (!sys_isuser(optarg)) {
-					fprintf(stderr, "%s: '%s' is not an "
-					                "existing user\n",
-					                argv0, optarg);
-					exit(EXIT_FAILURE);
-				}
-				if (opts->dropuser)
-					free(opts->dropuser);
-				opts->dropuser = strdup(optarg);
-				if (!opts->dropuser)
-					oom_die(argv0);
+				opts_set_user(opts, argv0, optarg);
 				break;
 			case 'm':
-				if (!sys_isgroup(optarg)) {
-					fprintf(stderr, "%s: '%s' is not an "
-					                "existing group\n",
-					                argv0, optarg);
-					exit(EXIT_FAILURE);
-				}
-				if (opts->dropgroup)
-					free(opts->dropgroup);
-				opts->dropgroup = strdup(optarg);
-				if (!opts->dropgroup)
-					oom_die(argv0);
+				opts_set_group(opts, argv0, optarg);
 				break;
 			case 'p':
-				if (opts->pidfile)
-					free(opts->pidfile);
-				opts->pidfile = strdup(optarg);
-				if (!opts->pidfile)
-					oom_die(argv0);
+				opts_set_pidfile(opts, argv0, optarg);
 				break;
 			case 'j':
-				if (!sys_isdir(optarg)) {
-					fprintf(stderr, "%s: '%s' is not a "
-					                "directory\n",
-					                argv0, optarg);
-					exit(EXIT_FAILURE);
-				}
-				if (opts->jaildir)
-					free(opts->jaildir);
-				opts->jaildir = realpath(optarg, NULL);
-				if (!opts->jaildir) {
-					fprintf(stderr, "%s: Failed to "
-					                "canonicalize '%s': "
-					                "%s (%i)\n",
-					                argv0, optarg,
-					                strerror(errno), errno);
-					exit(EXIT_FAILURE);
-				}
+				opts_set_jaildir(opts, argv0, optarg);
 				break;
 			case 'l':
-				if (opts->connectlog)
-					free(opts->connectlog);
-				opts->connectlog = strdup(optarg);
-				if (!opts->connectlog)
-					oom_die(argv0);
+				opts_set_connectlog(opts, argv0, optarg);
 				break;
 			case 'L':
-				if (opts->contentlog)
-					free(opts->contentlog);
-				opts->contentlog = strdup(optarg);
-				if (!opts->contentlog)
-					oom_die(argv0);
-				opts->contentlog_isdir = 0;
-				opts->contentlog_isspec = 0;
+				opts_set_contentlog(opts, argv0, optarg);
 				break;
 			case 'S':
-				if (!sys_isdir(optarg)) {
-					fprintf(stderr, "%s: '%s' is not a "
-					                "directory\n",
-					                argv0, optarg);
-					exit(EXIT_FAILURE);
-				}
-				if (opts->contentlog)
-					free(opts->contentlog);
-				opts->contentlog = realpath(optarg, NULL);
-				if (!opts->contentlog) {
-					fprintf(stderr, "%s: Failed to "
-					                "canonicalize '%s': "
-					                "%s (%i)\n",
-					                argv0, optarg,
-					                strerror(errno), errno);
-					exit(EXIT_FAILURE);
-				}
-				opts->contentlog_isdir = 1;
-				opts->contentlog_isspec = 0;
+				opts_set_contentlogdir(opts, argv0, optarg);
 				break;
-			case 'F': {
-				char *lhs, *rhs, *p, *q;
-				size_t n;
-				if (opts->contentlog_basedir)
-					free(opts->contentlog_basedir);
-				if (opts->contentlog)
-					free(opts->contentlog);
-				if (log_content_split_pathspec(optarg, &lhs,
-				                               &rhs) == -1) {
-					fprintf(stderr, "%s: Failed to split "
-					                "'%s' in lhs/rhs: "
-					                "%s (%i)\n",
-					                argv0, optarg,
-					                strerror(errno), errno);
-					exit(EXIT_FAILURE);
-				}
-				/* eliminate %% from lhs */
-				for (p = q = lhs; *p; p++, q++) {
-					if (q < p)
-						*q = *p;
-					if (*p == '%' && *(p+1) == '%')
-						p++;
-				}
-				*q = '\0';
-				/* all %% in lhs resolved to % */
-				if (sys_mkpath(lhs, 0777) == -1) {
-					fprintf(stderr, "%s: Failed to create "
-					                "'%s': %s (%i)\n",
-					                argv0, lhs,
-					                strerror(errno), errno);
-					exit(EXIT_FAILURE);
-				}
-				opts->contentlog_basedir = realpath(lhs, NULL);
-				if (!opts->contentlog_basedir) {
-					fprintf(stderr, "%s: Failed to "
-					                "canonicalize '%s': "
-					                "%s (%i)\n",
-					                argv0, lhs,
-					                strerror(errno), errno);
-					exit(EXIT_FAILURE);
-				}
-				/* count '%' in opts->contentlog_basedir */
-				for (n = 0, p = opts->contentlog_basedir;
-				     *p;
-				     p++) {
-					if (*p == '%')
-						n++;
-				}
-				free(lhs);
-				n += strlen(opts->contentlog_basedir);
-				if (!(lhs = malloc(n + 1)))
-					oom_die(argv0);
-				/* re-encoding % to %%, copying basedir to lhs */
-				for (p = opts->contentlog_basedir, q = lhs;
-				     *p;
-				     p++, q++) {
-					*q = *p;
-					if (*q == '%')
-						*(++q) = '%';
-				}
-				*q = '\0';
-				/* lhs contains encoded realpathed basedir */
-				if (asprintf(&opts->contentlog,
-				             "%s/%s", lhs, rhs) < 0)
-					oom_die(argv0);
-				opts->contentlog_isdir = 0;
-				opts->contentlog_isspec = 1;
-				free(lhs);
-				free(rhs);
+			case 'F':
+				opts_set_contentlogpathspec(opts, argv0, optarg);
 				break;
+			case 'X':
+				opts_set_pcaplog(opts, argv0, optarg);
+				break;
+			case 'Y':
+				opts_set_pcaplogdir(opts, argv0, optarg);
+				break;
+			case 'y':
+				opts_set_pcaplogpathspec(opts, argv0, optarg);
+				break;
+#ifndef WITHOUT_MIRROR
+			case 'I':
+				opts_set_mirrorif(opts, argv0, optarg);
+				break;
+			case 'T':
+				opts_set_mirrortarget(opts, argv0, optarg);
+				break;
+#endif /* !WITHOUT_MIRROR */
 			case 'W':
-				opts->certgen_writeall = 1;
-				if (opts->certgendir)
-					free(opts->certgendir);
-				opts->certgendir = strdup(optarg);
-				if (!opts->certgendir)
-					oom_die(argv0);
+				opts_set_certgendir_writeall(opts, argv0, optarg);
 				break;
 			case 'w':
-				opts->certgen_writeall = 0;
-				if (opts->certgendir)
-					free(opts->certgendir);
-				opts->certgendir = strdup(optarg);
-				if (!opts->certgendir)
-					oom_die(argv0);
+				opts_set_certgendir_writegencerts(opts, argv0, optarg);
 				break;
-			}
 #ifdef HAVE_LOCAL_PROCINFO
 			case 'i':
-				opts->lprocinfo = 1;
+				opts_set_lprocinfo(opts);
 				break;
 #endif /* HAVE_LOCAL_PROCINFO */
 			case 'M':
-				if (opts->masterkeylog)
-					free(opts->masterkeylog);
-				opts->masterkeylog = strdup(optarg);
-				if (!opts->masterkeylog)
-					oom_die(argv0);
+				opts_set_masterkeylog(opts, argv0, optarg);
 				break;
 			case 'd':
-				opts->detach = 1;
+				opts_set_daemon(opts);
 				break;
 			case 'D':
-				log_dbg_mode(LOG_DBG_MODE_ERRLOG);
-				opts->debug = 1;
+				opts_set_debug(opts);
 				break;
 			case 'V':
 				main_version();
@@ -741,7 +511,7 @@ main(int argc, char *argv[])
 	}
 	argc -= optind;
 	argv += optind;
-	opts->spec = proxyspec_parse(&argc, &argv, natengine);
+	proxyspec_parse(&argc, &argv, natengine, &opts->spec);
 
 	/* usage checks before defaults */
 	if (opts->detach && OPTS_DEBUG(opts)) {
@@ -749,6 +519,16 @@ main(int argc, char *argv[])
 		                argv0);
 		exit(EXIT_FAILURE);
 	}
+#ifndef WITHOUT_MIRROR
+	if (opts->mirrortarget && !opts->mirrorif) {
+		fprintf(stderr, "%s: -T depends on -I.\n", argv0);
+		exit(EXIT_FAILURE);
+	}
+	if (opts->mirrorif && !opts->mirrortarget) {
+		fprintf(stderr, "%s: -I depends on -T.\n", argv0);
+		exit(EXIT_FAILURE);
+	}
+#endif /* !WITHOUT_MIRROR */
 	if (!opts->spec) {
 		fprintf(stderr, "%s: no proxyspec specified.\n", argv0);
 		exit(EXIT_FAILURE);
@@ -778,6 +558,14 @@ main(int argc, char *argv[])
 			                argv0);
 			exit(EXIT_FAILURE);
 		}
+#ifndef OPENSSL_NO_ENGINE
+		if (opts->openssl_engine &&
+		    ssl_engine(opts->openssl_engine) == -1) {
+			fprintf(stderr, "%s: failed to enable OpenSSL engine"
+			                " %s.\n", argv0, opts->openssl_engine);
+			exit(EXIT_FAILURE);
+		}
+#endif /* !OPENSSL_NO_ENGINE */
 		if ((opts->cacrt || !opts->tgcrtdir) && !opts->cakey) {
 			fprintf(stderr, "%s: no CA key specified (-k).\n",
 			                argv0);
@@ -837,16 +625,69 @@ main(int argc, char *argv[])
 		}
 #endif /* __APPLE__ */
 	}
+	if (opts->dropuser && sys_isgeteuid(opts->dropuser)) {
+		if (opts->dropgroup) {
+			fprintf(stderr, "%s: cannot use -m when -u is "
+			        "current user\n", argv0);
+			exit(EXIT_FAILURE);
+		}
+		free(opts->dropuser);
+		opts->dropuser = NULL;
+	}
+
+	/* usage checks after defaults */
+	if (opts->dropgroup && !opts->dropuser) {
+		fprintf(stderr, "%s: -m depends on -u\n", argv0);
+		exit(EXIT_FAILURE);
+	}
+
+	/* Warn about options that require per-connection privileged operations
+	 * to be executed through privsep, but only if dropuser is set and is
+	 * not root, because privsep will fastpath in that situation, skipping
+	 * the latency-incurring overhead. */
+	int privsep_warn = 0;
+	if (opts->dropuser) {
+		if (opts->contentlog_isdir) {
+			log_dbg_printf("| Warning: -F requires a privileged "
+			               "operation for each connection!\n");
+			privsep_warn = 1;
+		}
+		if (opts->contentlog_isspec) {
+			log_dbg_printf("| Warning: -S requires a privileged "
+			               "operation for each connection!\n");
+			privsep_warn = 1;
+		}
+		if (opts->pcaplog_isdir) {
+			log_dbg_printf("| Warning: -Y requires a privileged "
+			               "operation for each connection!\n");
+			privsep_warn = 1;
+		}
+		if (opts->pcaplog_isspec) {
+			log_dbg_printf("| Warning: -y requires a privileged "
+			               "operation for each connection!\n");
+			privsep_warn = 1;
+		}
+		if (opts->certgendir) {
+			log_dbg_printf("| Warning: -w/-W require a privileged "
+			               "op for each connection!\n");
+			privsep_warn = 1;
+		}
+	}
+	if (privsep_warn) {
+		log_dbg_printf("| Privileged operations require communication "
+		               "between parent and child process\n"
+		               "| and will negatively impact latency and "
+		               "performance on each connection.\n");
+	}
+
+	/* debug log, part 1 */
+	if (OPTS_DEBUG(opts)) {
+		main_version();
+	}
+
+	/* generate leaf key */
 	if (opts_has_ssl_spec(opts) && opts->cakey && !opts->key) {
-		/*
-		 * While browsers still generally accept it, use a leaf key
-		 * size of 1024 bit for leaf keys.  When browsers start to
-		 * sunset 1024 bit RSA in leaf keys, we will need to make this
-		 * value bigger, and/or configurable.  Until then, users who
-		 * want a different size can always use their own pre-generated
-		 * leaf key instead of generating one.
-		 */
-		opts->key = ssl_key_genrsa(1024);
+		opts->key = ssl_key_genrsa(DFLT_LEAFKEY_RSABITS);
 		if (!opts->key) {
 			fprintf(stderr, "%s: error generating RSA key:\n",
 			                argv0);
@@ -857,7 +698,6 @@ main(int argc, char *argv[])
 			log_dbg_printf("Generated RSA key for leaf certs.\n");
 		}
 	}
-
 	if (opts->certgendir) {
 		char *keyid, *keyfn;
 		int prv;
@@ -892,15 +732,8 @@ main(int argc, char *argv[])
 		fclose(keyf);
 	}
 
-	/* usage checks after defaults */
-	if (opts->dropgroup && !opts->dropuser) {
-		fprintf(stderr, "%s: -m depends on -u.\n", argv0);
-		exit(EXIT_FAILURE);
-	}
-
-	/* debugging */
+	/* debug log, part 2 */
 	if (OPTS_DEBUG(opts)) {
-		main_version();
 		opts_proto_dbg_dump(opts);
 		log_dbg_printf("proxyspecs:\n");
 		for (proxyspec_t *spec = opts->spec; spec; spec = spec->next) {
@@ -912,6 +745,12 @@ main(int argc, char *argv[])
 			log_dbg_printf("- %s\n", specstr);
 			free(specstr);
 		}
+#ifndef OPENSSL_NO_ENGINE
+		if (opts->openssl_engine) {
+			log_dbg_printf("Loaded OpenSSL engine %s\n",
+			               opts->openssl_engine);
+		}
+#endif /* !OPENSSL_NO_ENGINE */
 		if (opts->cacrt) {
 			char *subj = ssl_x509_subject(opts->cacrt);
 			log_dbg_printf("Loaded CA: '%s'\n", subj);
@@ -974,11 +813,13 @@ main(int argc, char *argv[])
 	}
 
 	/* Fork into parent monitor process and (potentially unprivileged)
-	 * child process doing the actual work.  We request 3 privsep client
-	 * sockets: content logger thread, cert writer thread, and the child
-	 * process main thread (main proxy thread) */
-	int clisock[3];
-	if (privsep_fork(opts, clisock, 3) != 0) {
+	 * child process doing the actual work.  We request 6 privsep client
+	 * sockets: five logger threads, and the child process main thread,
+	 * which will become the main proxy thread.  First slot is main thread,
+	 * remaining slots are passed down to log subsystem. */
+	int clisock[6];
+	if (privsep_fork(opts, clisock,
+	                 sizeof(clisock)/sizeof(clisock[0])) != 0) {
 		/* parent has exited the monitor loop after waiting for child,
 		 * or an error occured */
 		if (opts->pidfile) {
@@ -1006,13 +847,17 @@ main(int argc, char *argv[])
 		               strerror(errno), errno);
 		exit(EXIT_FAILURE);
 	}
+	log_dbg_printf("Dropped privs to user %s group %s chroot %s\n",
+	               opts->dropuser  ? opts->dropuser  : "-",
+	               opts->dropgroup ? opts->dropgroup : "-",
+	               opts->jaildir   ? opts->jaildir   : "-");
 	if (ssl_reinit() == -1) {
 		fprintf(stderr, "%s: failed to reinit SSL\n", argv0);
 		goto out_sslreinit_failed;
 	}
 
 	/* Post-privdrop/chroot/detach initialization, thread spawning */
-	if (log_init(opts, proxy, clisock[1], clisock[2]) == -1) {
+	if (log_init(opts, proxy, &clisock[1]) == -1) {
 		fprintf(stderr, "%s: failed to init log facility: %s\n",
 		                argv0, strerror(errno));
 		goto out_log_failed;
