@@ -38,9 +38,11 @@
 static char *argv01[] = {
 	"https", "127.0.0.1", "10443", "127.0.0.2", "443"
 };
+#ifndef TRAVIS
 static char *argv02[] = {
 	"https", "::1", "10443", "::2", "443"
 };
+#endif /* !TRAVIS */
 static char *argv03[] = {
 	"http", "127.0.0.1", "10443", "127.0.0.2", "443"
 };
@@ -53,9 +55,11 @@ static char *argv05[] = {
 static char *argv06[] = {
 	"https", "127.0.0.1", "10443", "sni", "443"
 };
+#ifndef DOCKER
 static char *argv07[] = {
 	"http", "127.0.0.1", "10443", "sni", "443"
 };
+#endif /* !DOCKER */
 static char *argv08[] = {
 	"https", "127.0.0.1", "10443", "no_such_engine"
 };
@@ -74,23 +78,29 @@ static char *argv12[] = {
 	"autossl", "127.0.0.1", "10025", "127.0.0.2", "25",
 	"https", "127.0.0.1", "10443", "127.0.0.2", "443"
 };
+#ifndef DOCKER
 static char *argv13[] = {
 	"autossl", "127.0.0.1", "10025", "sni", "25"
 };
+#endif /* !DOCKER */
 static char *argv14[] = {
 	"https", "127.0.0.1", "10443",
 	"autossl", "127.0.0.1", "10025", "127.0.0.2", "25"
 };
 
+#ifdef __linux__
+#define NATENGINE "netfilter"
+#else
 #define NATENGINE "pf"
+#endif
 
 START_TEST(proxyspec_parse_01)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv01;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -108,13 +118,14 @@ START_TEST(proxyspec_parse_01)
 }
 END_TEST
 
+#ifndef TRAVIS
 START_TEST(proxyspec_parse_02)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv02;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -131,40 +142,45 @@ START_TEST(proxyspec_parse_02)
 	proxyspec_free(spec);
 }
 END_TEST
+#endif /* !TRAVIS */
 
+#ifndef DOCKER
 START_TEST(proxyspec_parse_03)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 2;
 	char **argv = argv01;
 
 	close(2);
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	if (spec)
 		proxyspec_free(spec);
 }
 END_TEST
+#endif /* !DOCKER */
 
+#ifndef DOCKER
 START_TEST(proxyspec_parse_04)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 4;
 	char **argv = argv01;
 
 	close(2);
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	if (spec)
 		proxyspec_free(spec);
 }
 END_TEST
+#endif /* !DOCKER */
 
 START_TEST(proxyspec_parse_05)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv03;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(!spec->ssl, "SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -184,11 +200,11 @@ END_TEST
 
 START_TEST(proxyspec_parse_06)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv04;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(!spec->http, "HTTP");
@@ -208,11 +224,11 @@ END_TEST
 
 START_TEST(proxyspec_parse_07)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv05;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(!spec->ssl, "SSL");
 	fail_unless(!spec->http, "HTTP");
@@ -232,11 +248,11 @@ END_TEST
 
 START_TEST(proxyspec_parse_08)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv06;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -253,14 +269,15 @@ START_TEST(proxyspec_parse_08)
 }
 END_TEST
 
+#ifndef DOCKER
 START_TEST(proxyspec_parse_09)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv07;
 
 	close(2);
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	if (spec)
 		proxyspec_free(spec);
 }
@@ -268,24 +285,25 @@ END_TEST
 
 START_TEST(proxyspec_parse_10)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 4;
 	char **argv = argv06;
 
 	close(2);
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	if (spec)
 		proxyspec_free(spec);
 }
 END_TEST
+#endif /* !DOCKER */
 
 START_TEST(proxyspec_parse_11)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 3;
 	char **argv = argv08;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -303,26 +321,29 @@ START_TEST(proxyspec_parse_11)
 }
 END_TEST
 
+#ifndef DOCKER
 START_TEST(proxyspec_parse_12)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 4;
 	char **argv = argv08;
 
 	close(2);
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	if (spec)
 		proxyspec_free(spec);
 }
 END_TEST
+#endif /* !DOCKER */
 
+#ifndef TRAVIS
 START_TEST(proxyspec_parse_13)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 10;
 	char **argv = argv09;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -353,11 +374,11 @@ END_TEST
 
 START_TEST(proxyspec_parse_14)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 6;
 	char **argv = argv10;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -386,14 +407,15 @@ START_TEST(proxyspec_parse_14)
 	proxyspec_free(spec);
 }
 END_TEST
+#endif /* !TRAVIS */
 
 START_TEST(proxyspec_parse_15)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 3;
 	char **argv = argv11;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(!spec->ssl, "SSL");
 	fail_unless(!spec->http, "HTTP");
@@ -412,11 +434,11 @@ END_TEST
 
 START_TEST(proxyspec_parse_16)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 10;
 	char **argv = argv12;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(spec->ssl, "not SSL");
 	fail_unless(spec->http, "not HTTP");
@@ -445,26 +467,28 @@ START_TEST(proxyspec_parse_16)
 }
 END_TEST
 
+#ifndef DOCKER
 START_TEST(proxyspec_parse_17)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 5;
 	char **argv = argv13;
 
 	close(2);
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	if (spec)
 		proxyspec_free(spec);
 }
 END_TEST
+#endif /* !DOCKER */
 
 START_TEST(proxyspec_parse_18)
 {
-	proxyspec_t *spec;
+	proxyspec_t *spec = NULL;
 	int argc = 8;
 	char **argv = argv14;
 
-	spec = proxyspec_parse(&argc, &argv, NATENGINE);
+	proxyspec_parse(&argc, &argv, NATENGINE, &spec);
 	fail_unless(!!spec, "failed to parse spec");
 	fail_unless(!spec->ssl, "SSL");
 	fail_unless(!spec->http, "HTTP");
@@ -518,24 +542,32 @@ opts_suite(void)
 	tcase_add_test(tc, proxyspec_parse_01);
 #ifndef TRAVIS
 	tcase_add_test(tc, proxyspec_parse_02); /* IPv6 */
-#endif /* TRAVIS */
+#endif /* !TRAVIS */
+#ifndef DOCKER
 	tcase_add_exit_test(tc, proxyspec_parse_03, EXIT_FAILURE);
 	tcase_add_exit_test(tc, proxyspec_parse_04, EXIT_FAILURE);
+#endif /* !DOCKER */
 	tcase_add_test(tc, proxyspec_parse_05);
 	tcase_add_test(tc, proxyspec_parse_06);
 	tcase_add_test(tc, proxyspec_parse_07);
 	tcase_add_test(tc, proxyspec_parse_08);
+#ifndef DOCKER
 	tcase_add_exit_test(tc, proxyspec_parse_09, EXIT_FAILURE);
 	tcase_add_exit_test(tc, proxyspec_parse_10, EXIT_FAILURE);
+#endif /* !DOCKER */
 	tcase_add_test(tc, proxyspec_parse_11);
+#ifndef DOCKER
 	tcase_add_exit_test(tc, proxyspec_parse_12, EXIT_FAILURE);
+#endif /* !DOCKER */
 #ifndef TRAVIS
 	tcase_add_test(tc, proxyspec_parse_13); /* IPv6 */
 	tcase_add_test(tc, proxyspec_parse_14); /* IPv6 */
-#endif /* TRAVIS */
+#endif /* !TRAVIS */
 	tcase_add_test(tc, proxyspec_parse_15);
 	tcase_add_test(tc, proxyspec_parse_16);
+#ifndef DOCKER
 	tcase_add_exit_test(tc, proxyspec_parse_17, EXIT_FAILURE);
+#endif /* !DOCKER */
 	tcase_add_test(tc, proxyspec_parse_18);
 	suite_add_tcase(s, tc);
 
@@ -543,6 +575,13 @@ opts_suite(void)
 	tcase_add_test(tc, opts_debug_01);
 	suite_add_tcase(s, tc);
 
+#ifdef DOCKER
+	fprintf(stderr, "opts: 6 tests omitted because building in docker\n");
+#endif
+#ifdef TRAVIS
+	fprintf(stderr, "opts: 3 tests omitted because building in travis\n");
+#endif
+
 	return s;
 }
 
