@@ -97,9 +97,6 @@ typedef struct pxy_conn_desc {
 #ifdef HAVE_LOCAL_PROCINFO
 /* local process data - filled in iff pid != -1 */
 typedef struct pxy_conn_lproc_desc {
-	struct sockaddr_storage srcaddr;
-	socklen_t srcaddrlen;
-
 	pid_t pid;
 	uid_t uid;
 	gid_t gid;
@@ -166,15 +163,17 @@ typedef struct pxy_conn_ctx {
 #endif /* HAVE_LOCAL_PROCINFO */
 
 	/* content log context */
-	log_content_ctx_t *logctx;
+	log_content_ctx_t logctx;
 
 	/* store fd and fd event while connected is 0 */
 	evutil_socket_t fd;
 	struct event *ev;
 
-	/* original destination address, family and certificate */
-	struct sockaddr_storage addr;
-	socklen_t addrlen;
+	/* original source and destination address, family and certificate */
+	struct sockaddr_storage srcaddr;
+	socklen_t srcaddrlen;
+	struct sockaddr_storage dstaddr;
+	socklen_t dstaddrlen;
 	int af;
 	X509 *origcrt;
 
@@ -188,7 +187,11 @@ typedef struct pxy_conn_ctx {
 } pxy_conn_ctx_t;
 
 #define WANT_CONNECT_LOG(ctx)	((ctx)->opts->connectlog||!(ctx)->opts->detach)
-#define WANT_CONTENT_LOG(ctx)	((ctx)->opts->contentlog&&!(ctx)->passthrough)
+#ifndef WITHOUT_MIRROR
+#define WANT_CONTENT_LOG(ctx)	(((ctx)->opts->contentlog||(ctx)->opts->pcaplog||(ctx)->opts->mirrorif)&&!(ctx)->passthrough)
+#else /* WITHOUT_MIRROR */
+#define WANT_CONTENT_LOG(ctx)	(((ctx)->opts->contentlog||(ctx)->opts->pcaplog)&&!(ctx)->passthrough)
+#endif /* WITHOUT_MIRROR */
 
 static pxy_conn_ctx_t *
 pxy_conn_ctx_new(proxyspec_t *spec, opts_t *opts,
@@ -229,7 +232,7 @@ pxy_conn_ctx_free(pxy_conn_ctx_t *ctx, int by_requestor)
 		                (void*)ctx);
 	}
 #endif /* DEBUG_PROXY */
-	if (WANT_CONTENT_LOG(ctx) && ctx->logctx) {
+	if (WANT_CONTENT_LOG(ctx)) {
 		if (log_content_close(&ctx->logctx, by_requestor) == -1) {
 			log_err_printf("Warning: Content log close failed\n");
 		}
@@ -313,7 +316,7 @@ static int pxy_ossl_servername_cb(SSL *ssl, int *al, void *arg);
 #endif /* !OPENSSL_NO_TLSEXT */
 static int pxy_ossl_sessnew_cb(SSL *, SSL_SESSION *);
 static void pxy_ossl_sessremove_cb(SSL_CTX *, SSL_SESSION *);
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x20800000L)
 static SSL_SESSION * pxy_ossl_sessget_cb(SSL *, unsigned char *, int, int *);
 #else /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
 static SSL_SESSION * pxy_ossl_sessget_cb(SSL *, const unsigned char *, int,
@@ -374,7 +377,7 @@ pxy_log_connect_nonhttp(pxy_conn_ctx_t *ctx)
 			goto out;
 		}
 	} else {
-		lpi = "";
+		lpi = "-";
 	}
 #endif /* HAVE_LOCAL_PROCINFO */
 
@@ -478,6 +481,8 @@ pxy_log_connect_http(pxy_conn_ctx_t *ctx)
 			ctx->enomem = 1;
 			goto out;
 		}
+	} else {
+		lpi = "-";
 	}
 #endif /* HAVE_LOCAL_PROCINFO */
 
@@ -554,7 +559,7 @@ pxy_log_connect_http(pxy_conn_ctx_t *ctx)
 	}
 out:
 #ifdef HAVE_LOCAL_PROCINFO
-	if (lpi) {
+	if (lpi && ctx->opts->lprocinfo) {
 		free(lpi);
 	}
 #endif /* HAVE_LOCAL_PROCINFO */
@@ -625,7 +630,7 @@ pxy_ossl_sessremove_cb(UNUSED SSL_CTX *sslctx, SSL_SESSION *sess)
  * Called by OpenSSL when a src SSL session is requested by the client.
  */
 static SSL_SESSION *
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x20800000L)
 pxy_ossl_sessget_cb(UNUSED SSL *ssl, unsigned char *id, int idlen,
                     int *copy)
 #else /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
@@ -720,12 +725,14 @@ pxy_srcsslctx_create(pxy_conn_ctx_t *ctx, X509 *crt, STACK_OF(X509) *chain,
                      EVP_PKEY *key)
 {
 	SSL_CTX *sslctx = SSL_CTX_new(ctx->opts->sslmethod());
-	if (!sslctx)
+	if (!sslctx) {
+		ctx->enomem = 1;
 		return NULL;
+	}
 
 	pxy_sslctx_setoptions(sslctx, ctx);
 
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L) && !defined(LIBRESSL_VERSION_NUMBER)
 	if (ctx->opts->sslversion) {
 		if (SSL_CTX_set_min_proto_version(sslctx, ctx->opts->sslversion) == 0 ||
 			SSL_CTX_set_max_proto_version(sslctx, ctx->opts->sslversion) == 0) {
@@ -766,8 +773,16 @@ pxy_srcsslctx_create(pxy_conn_ctx_t *ctx, X509 *crt, STACK_OF(X509) *chain,
 		EC_KEY_free(ecdh);
 	}
 #endif /* !OPENSSL_NO_ECDH */
-	SSL_CTX_use_certificate(sslctx, crt);
-	SSL_CTX_use_PrivateKey(sslctx, key);
+	if (SSL_CTX_use_certificate(sslctx, crt) != 1) {
+		log_dbg_printf("loading src server certificate failed\n");
+		SSL_CTX_free(sslctx);
+		return NULL;
+	}
+	if (SSL_CTX_use_PrivateKey(sslctx, key) != 1) {
+		log_dbg_printf("loading src server key failed\n");
+		SSL_CTX_free(sslctx);
+		return NULL;
+	}
 	for (int i = 0; i < sk_X509_num(chain); i++) {
 		X509 *c = sk_X509_value(chain, i);
 		ssl_x509_refcount_inc(c); /* next call consumes a reference */
@@ -940,8 +955,8 @@ pxy_srcssl_create(pxy_conn_ctx_t *ctx, SSL *origssl)
 {
 	cert_t *cert;
 
-	cachemgr_dsess_set((struct sockaddr*)&ctx->addr,
-	                   ctx->addrlen, ctx->sni,
+	cachemgr_dsess_set((struct sockaddr*)&ctx->dstaddr,
+	                   ctx->dstaddrlen, ctx->sni,
 	                   SSL_get0_session(origssl));
 
 	ctx->origcrt = SSL_get_peer_certificate(origssl);
@@ -975,10 +990,8 @@ pxy_srcssl_create(pxy_conn_ctx_t *ctx, SSL *origssl)
 	SSL_CTX *sslctx = pxy_srcsslctx_create(ctx, cert->crt, cert->chain,
 	                                       cert->key);
 	cert_free(cert);
-	if (!sslctx) {
-		ctx->enomem = 1;
+	if (!sslctx)
 		return NULL;
-	}
 	SSL *ssl = SSL_new(sslctx);
 	SSL_CTX_free(sslctx); /* SSL_new() increments refcount */
 	if (!ssl) {
@@ -1044,7 +1057,7 @@ pxy_ossl_servername_cb(SSL *ssl, UNUSED int *al, void *arg)
 
 	/* generate a new certificate with sn as additional altSubjectName
 	 * and replace it both in the current SSL ctx and in the cert cache */
-	if (!ctx->immutable_cert &&
+	if (ctx->opts->allow_wrong_host && !ctx->immutable_cert &&
 	    !ssl_x509_names_match((sslcrt = SSL_get_certificate(ssl)), sn)) {
 		X509 *newcrt;
 		SSL_CTX *newsslctx;
@@ -1090,7 +1103,6 @@ pxy_ossl_servername_cb(SSL *ssl, UNUSED int *al, void *arg)
 		                                 ctx->opts->key);
 		if (!newsslctx) {
 			X509_free(newcrt);
-			ctx->enomem = 1;
 			return SSL_TLSEXT_ERR_NOACK;
 		}
 		SSL_set_SSL_CTX(ssl, newsslctx); /* decr's old incr new refc */
@@ -1124,26 +1136,34 @@ pxy_dstssl_create(pxy_conn_ctx_t *ctx)
 
 	pxy_sslctx_setoptions(sslctx, ctx);
 
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L) && !defined(LIBRESSL_VERSION_NUMBER)
 	if (ctx->opts->sslversion) {
 		if (SSL_CTX_set_min_proto_version(sslctx, ctx->opts->sslversion) == 0 ||
 			SSL_CTX_set_max_proto_version(sslctx, ctx->opts->sslversion) == 0) {
 			SSL_CTX_free(sslctx);
-			ctx->enomem = 1;
 			return NULL;
 		}
 	}
 #endif /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
 
-	SSL_CTX_set_verify(sslctx, SSL_VERIFY_NONE, NULL);
-
-	if (ctx->opts->clientcrt) {
-		if (!SSL_CTX_use_certificate(sslctx, ctx->opts->clientcrt))
-			log_dbg_printf("loading client certificate failed");
+	if (ctx->opts->verify_peer) {
+		SSL_CTX_set_verify(sslctx, SSL_VERIFY_PEER, NULL);
+		SSL_CTX_set_default_verify_paths(sslctx);
+	} else {
+		SSL_CTX_set_verify(sslctx, SSL_VERIFY_NONE, NULL);
 	}
-	if (ctx->opts->clientkey) {
-		if (!SSL_CTX_use_PrivateKey(sslctx, ctx->opts->clientkey))
-			log_dbg_printf("loading client key failed");
+
+	if (ctx->opts->clientcrt &&
+	    (SSL_CTX_use_certificate(sslctx, ctx->opts->clientcrt) != 1)) {
+		log_dbg_printf("loading dst client certificate failed\n");
+		SSL_CTX_free(sslctx);
+		return NULL;
+	}
+	if (ctx->opts->clientkey &&
+	    (SSL_CTX_use_PrivateKey(sslctx, ctx->opts->clientkey) != 1)) {
+		log_dbg_printf("loading dst client key failed\n");
+		SSL_CTX_free(sslctx);
+		return NULL;
 	}
 
 	ssl = SSL_new(sslctx);
@@ -1164,8 +1184,8 @@ pxy_dstssl_create(pxy_conn_ctx_t *ctx)
 #endif /* SSL_MODE_RELEASE_BUFFERS */
 
 	/* session resuming based on remote endpoint address and port */
-	sess = cachemgr_dsess_get((struct sockaddr *)&ctx->addr,
-	                          ctx->addrlen, ctx->sni); /* new sess inst */
+	sess = cachemgr_dsess_get((struct sockaddr *)&ctx->dstaddr,
+	                          ctx->dstaddrlen, ctx->sni);
 	if (sess) {
 		if (OPTS_DEBUG(ctx->opts)) {
 			log_dbg_printf("Attempt reuse dst SSL session\n");
@@ -1409,6 +1429,10 @@ pxy_http_resphdr_filter_line(const char *line, pxy_conn_ctx_t *ctx)
 		    /* HSTS: HTTP Strict Transport Security (RFC 6797)
 		     * remove to allow users to accept bad certs */
 		    !strncasecmp(line, "Strict-Transport-Security:", 26) ||
+		    /* Expect-CT: Expect Certificate Transparency
+		     * (draft-ietf-httpbis-expect-ct-latest)
+		     * remove to prevent failed CT log lookups */
+		    !strncasecmp(line, "Expect-CT:", 10) ||
 		    /* Alternate Protocol
 		     * remove to prevent switching to QUIC, SPDY et al */
 		    !strncasecmp(line, "Alternate-Protocol:", 19) ||
@@ -1517,11 +1541,10 @@ deny:
 	if (evbuffer_get_length(inbuf) > 0) {
 		if (WANT_CONTENT_LOG(ctx)) {
 			logbuf_t *lb;
-			lb = logbuf_new_alloc(evbuffer_get_length(inbuf),
-			                      NULL, NULL);
+			lb = logbuf_new_alloc(evbuffer_get_length(inbuf), NULL);
 			if (lb &&
 			    (evbuffer_copyout(inbuf, lb->buf, lb->sz) != -1)) {
-				if (log_content_submit(ctx->logctx, lb,
+				if (log_content_submit(&ctx->logctx, lb,
 				                       1/*req*/) == -1) {
 					logbuf_free(lb);
 					log_err_printf("Warning: Content log "
@@ -1538,10 +1561,9 @@ deny:
 	ctx->ocsp_denied = 1;
 	if (WANT_CONTENT_LOG(ctx)) {
 		logbuf_t *lb;
-		lb = logbuf_new_copy(ocspresp, sizeof(ocspresp) - 1,
-		                     NULL, NULL);
+		lb = logbuf_new_copy(ocspresp, sizeof(ocspresp) - 1, NULL);
 		if (lb) {
-			if (log_content_submit(ctx->logctx, lb,
+			if (log_content_submit(&ctx->logctx, lb,
 			                       0/*resp*/) == -1) {
 				logbuf_free(lb);
 				log_err_printf("Warning: Content log "
@@ -1684,8 +1706,7 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 			char *replace;
 			if (WANT_CONTENT_LOG(ctx)) {
 				logbuf_t *tmp;
-				tmp = logbuf_new_printf(NULL, NULL,
-				                        "%s\r\n", line);
+				tmp = logbuf_new_printf(NULL, "%s\r\n", line);
 				if (tail) {
 					if (tmp) {
 						tail->next = tmp;
@@ -1712,7 +1733,7 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 			}
 		}
 		if (lb && WANT_CONTENT_LOG(ctx)) {
-			if (log_content_submit(ctx->logctx, lb,
+			if (log_content_submit(&ctx->logctx, lb,
 			                       1/*req*/) == -1) {
 				logbuf_free(lb);
 				log_err_printf("Warning: Content log "
@@ -1732,8 +1753,7 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 			char *replace;
 			if (WANT_CONTENT_LOG(ctx)) {
 				logbuf_t *tmp;
-				tmp = logbuf_new_printf(NULL, NULL,
-				                        "%s\r\n", line);
+				tmp = logbuf_new_printf(NULL, "%s\r\n", line);
 				if (tail) {
 					if (tmp) {
 						tail->next = tmp;
@@ -1760,7 +1780,7 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 			}
 		}
 		if (lb && WANT_CONTENT_LOG(ctx)) {
-			if (log_content_submit(ctx->logctx, lb,
+			if (log_content_submit(&ctx->logctx, lb,
 			                       0/*resp*/) == -1) {
 				logbuf_free(lb);
 				log_err_printf("Warning: Content log "
@@ -1783,9 +1803,9 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 
 	if (WANT_CONTENT_LOG(ctx)) {
 		logbuf_t *lb;
-		lb = logbuf_new_alloc(evbuffer_get_length(inbuf), NULL, NULL);
+		lb = logbuf_new_alloc(evbuffer_get_length(inbuf), NULL);
 		if (lb && (evbuffer_copyout(inbuf, lb->buf, lb->sz) != -1)) {
-			if (log_content_submit(ctx->logctx, lb,
+			if (log_content_submit(&ctx->logctx, lb,
 			                       (bev == ctx->src.bev)) == -1) {
 				logbuf_free(lb);
 				log_err_printf("Warning: Content log "
@@ -1934,7 +1954,7 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 		/* prepare logging, part 2 */
 		if (WANT_CONNECT_LOG(ctx) || WANT_CONTENT_LOG(ctx)) {
 			if (sys_sockaddr_str((struct sockaddr *)
-			                     &ctx->addr, ctx->addrlen,
+			                     &ctx->dstaddr, ctx->dstaddrlen,
 			                     &ctx->dsthost_str,
 			                     &ctx->dstport_str) != 0) {
 				ctx->enomem = 1;
@@ -1946,8 +1966,8 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 			if (ctx->opts->lprocinfo) {
 				/* fetch process info */
 				if (proc_pid_for_addr(&ctx->lproc.pid,
-				        (struct sockaddr*)&ctx->lproc.srcaddr,
-				        ctx->lproc.srcaddrlen) == 0 &&
+				        (struct sockaddr*)&ctx->srcaddr,
+				        ctx->srcaddrlen) == 0 &&
 				    ctx->lproc.pid != -1 &&
 				    proc_get_info(ctx->lproc.pid,
 				                  &ctx->lproc.exec_path,
@@ -1970,6 +1990,10 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 		}
 		if (WANT_CONTENT_LOG(ctx)) {
 			if (log_content_open(&ctx->logctx, ctx->opts,
+			                     (struct sockaddr *)&ctx->srcaddr,
+			                     ctx->srcaddrlen,
+			                     (struct sockaddr *)&ctx->dstaddr,
+			                     ctx->dstaddrlen,
 			                     ctx->srchost_str, ctx->srcport_str,
 			                     ctx->dsthost_str, ctx->dstport_str,
 #ifdef HAVE_LOCAL_PROCINFO
@@ -2219,7 +2243,8 @@ leave:
 	if (OPTS_DEBUG(ctx->opts)) {
 		log_dbg_printf("%s disconnected to [%s]:%s\n",
 		               this->ssl ? "SSL" : "TCP",
-		               ctx->dsthost_str, ctx->dstport_str);
+		               STRORDASH(ctx->dsthost_str),
+		               STRORDASH(ctx->dstport_str));
 		log_dbg_printf("%s disconnected from [%s]:%s\n",
 		               this->ssl ? "SSL" : "TCP",
 		               ctx->srchost_str, ctx->srcport_str);
@@ -2240,7 +2265,7 @@ leave:
 static void
 pxy_conn_connect(pxy_conn_ctx_t *ctx)
 {
-	if (!ctx->addrlen) {
+	if (!ctx->dstaddrlen) {
 		log_err_printf("No target address; aborting connection\n");
 		evutil_closesocket(ctx->fd);
 		pxy_conn_ctx_free(ctx, 1);
@@ -2270,8 +2295,8 @@ pxy_conn_connect(pxy_conn_ctx_t *ctx)
 
 	if (OPTS_DEBUG(ctx->opts)) {
 		char *host, *port;
-		if (sys_sockaddr_str((struct sockaddr *)&ctx->addr,
-		                     ctx->addrlen, &host, &port) != 0) {
+		if (sys_sockaddr_str((struct sockaddr *)&ctx->dstaddr,
+		                     ctx->dstaddrlen, &host, &port) != 0) {
 			log_dbg_printf("Connecting to [?]:?\n");
 		} else {
 			log_dbg_printf("Connecting to [%s]:%s\n", host, port);
@@ -2282,8 +2307,8 @@ pxy_conn_connect(pxy_conn_ctx_t *ctx)
 
 	/* initiate connection */
 	bufferevent_socket_connect(ctx->dst.bev,
-	                           (struct sockaddr *)&ctx->addr,
-	                           ctx->addrlen);
+	                           (struct sockaddr *)&ctx->dstaddr,
+	                           ctx->dstaddrlen);
 }
 
 #ifndef OPENSSL_NO_TLSEXT
@@ -2304,8 +2329,8 @@ pxy_sni_resolve_cb(int errcode, struct evutil_addrinfo *ai, void *arg)
 		return;
 	}
 
-	memcpy(&ctx->addr, ai->ai_addr, ai->ai_addrlen);
-	ctx->addrlen = ai->ai_addrlen;
+	memcpy(&ctx->dstaddr, ai->ai_addr, ai->ai_addrlen);
+	ctx->dstaddrlen = ai->ai_addrlen;
 	evutil_freeaddrinfo(ai);
 	pxy_conn_connect(ctx);
 }
@@ -2395,7 +2420,7 @@ pxy_fd_readcb(MAYBE_UNUSED evutil_socket_t fd, UNUSED short what, void *arg)
 		ctx->ev = NULL;
 	}
 
-	if (ctx->sni && !ctx->addrlen && ctx->spec->sni_port) {
+	if (ctx->sni && !ctx->dstaddrlen && ctx->spec->sni_port) {
 		char sniport[6];
 		struct evutil_addrinfo hints;
 
@@ -2447,8 +2472,9 @@ pxy_conn_setup(evutil_socket_t fd,
 	/* determine original destination of connection */
 	if (spec->natlookup) {
 		/* NAT engine lookup */
-		ctx->addrlen = sizeof(struct sockaddr_storage);
-		if (spec->natlookup((struct sockaddr *)&ctx->addr, &ctx->addrlen,
+		ctx->dstaddrlen = sizeof(struct sockaddr_storage);
+		if (spec->natlookup((struct sockaddr *)&ctx->dstaddr,
+		                    &ctx->dstaddrlen,
 		                    fd, peeraddr, peeraddrlen) == -1) {
 			log_err_printf("Connection not found in NAT "
 			               "state table, aborting connection\n");
@@ -2458,8 +2484,8 @@ pxy_conn_setup(evutil_socket_t fd,
 		}
 	} else if (spec->connect_addrlen > 0) {
 		/* static forwarding */
-		ctx->addrlen = spec->connect_addrlen;
-		memcpy(&ctx->addr, &spec->connect_addr, ctx->addrlen);
+		ctx->dstaddrlen = spec->connect_addrlen;
+		memcpy(&ctx->dstaddr, &spec->connect_addr, ctx->dstaddrlen);
 	} else {
 		/* SNI mode */
 		if (!ctx->spec->ssl) {
@@ -2473,17 +2499,22 @@ pxy_conn_setup(evutil_socket_t fd,
 	}
 
 	/* prepare logging, part 1 */
+	if (opts->pcaplog
+#ifndef WITHOUT_MIRROR
+	    || opts->mirrorif
+#endif /* !WITHOUT_MIRROR */
+#ifdef HAVE_LOCAL_PROCINFO
+	    || opts->lprocinfo
+#endif /* HAVE_LOCAL_PROCINFO */
+	    ) {
+		ctx->srcaddrlen = peeraddrlen;
+		memcpy(&ctx->srcaddr, peeraddr, ctx->srcaddrlen);
+	}
 	if (WANT_CONNECT_LOG(ctx) || WANT_CONTENT_LOG(ctx)) {
 		if (sys_sockaddr_str(peeraddr, peeraddrlen,
 		                     &ctx->srchost_str,
 		                     &ctx->srcport_str) != 0)
 			goto memout;
-#ifdef HAVE_LOCAL_PROCINFO
-		if (ctx->opts->lprocinfo) {
-			memcpy(&ctx->lproc.srcaddr, peeraddr, peeraddrlen);
-			ctx->lproc.srcaddrlen = peeraddrlen;
-		}
-#endif /* HAVE_LOCAL_PROCINFO */
 	}
 
 	/* for SSL, defer dst connection setup to initial_readcb */
