@@ -29,13 +29,22 @@
 #include "base64.h"
 #include "ssl.h"
 
+#include <limits.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include <check.h>
 
+#ifdef __APPLE__
+#define DLSUFFIX "dylib"
+#else
+#define DLSUFFIX "so"
+#endif
+
 #define TESTKEY "extra/pki/server.key"
 #define TESTCERT "extra/pki/server.crt"
 #define TESTCERT2 "extra/pki/rsa.crt"
+#define ENGINE "extra/engine/dummy-engine."DLSUFFIX
 
 static void
 ssl_setup(void)
@@ -548,6 +557,8 @@ START_TEST(ssl_key_identifier_sha1_01)
 	             "extension length mismatch");
 	fail_unless(!memcmp(ASN1_STRING_get0_data(value) + 2, keyid, SSL_KEY_IDSZ),
 	            "key id mismatch");
+	EVP_PKEY_free(k);
+	X509_free(c);
 }
 END_TEST
 
@@ -712,6 +723,55 @@ START_TEST(ssl_features_02)
 }
 END_TEST
 
+START_TEST(ssl_key_refcount_inc_01)
+{
+	EVP_PKEY *key;
+
+	key = ssl_key_load(TESTKEY);
+	fail_unless(!!key, "loading key failed");
+	ssl_key_refcount_inc(key);
+	ssl_key_refcount_inc(key);
+	ssl_key_refcount_inc(key);
+	EVP_PKEY_free(key);
+	/* these must not crash */
+	EVP_PKEY_free(key);
+	EVP_PKEY_free(key);
+	EVP_PKEY_free(key);
+}
+END_TEST
+
+START_TEST(ssl_x509_refcount_inc_01)
+{
+	X509 *crt;
+
+	crt = ssl_x509_load(TESTCERT);
+	fail_unless(!!crt, "loading certificate failed");
+	ssl_x509_refcount_inc(crt);
+	ssl_x509_refcount_inc(crt);
+	ssl_x509_refcount_inc(crt);
+	X509_free(crt);
+	/* these must not crash */
+	X509_free(crt);
+	X509_free(crt);
+	X509_free(crt);
+}
+END_TEST
+
+#ifndef OPENSSL_NO_ENGINE
+START_TEST(ssl_engine_01)
+{
+	char cwd[PATH_MAX];
+	char *path;
+
+	fail_unless(getcwd(cwd, sizeof(cwd)) == cwd, "getcwd() failed");
+	fail_unless(asprintf(&path, "%s/"ENGINE, cwd) != -1 && !!path,
+	            "constructing engine path failed");
+	fail_unless(ssl_engine(path) == 0, "loading OpenSSL engine failed");
+	free(path);
+}
+END_TEST
+#endif /* !OPENSSL_NO_ENGINE */
+
 Suite *
 ssl_suite(void)
 {
@@ -721,12 +781,14 @@ ssl_suite(void)
 	s = suite_create("ssl");
 
 	tc = tcase_create("ssl_wildcardify");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
 	tcase_add_test(tc, ssl_wildcardify_01);
 	tcase_add_test(tc, ssl_wildcardify_02);
 	tcase_add_test(tc, ssl_wildcardify_03);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ssl_dnsname_match");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
 	tcase_add_test(tc, ssl_dnsname_match_01);
 	tcase_add_test(tc, ssl_dnsname_match_02);
 	tcase_add_test(tc, ssl_dnsname_match_03);
@@ -746,6 +808,7 @@ ssl_suite(void)
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ssl_tls_clienthello_parse");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
 	tcase_add_test(tc, ssl_tls_clienthello_parse_00);
 	tcase_add_test(tc, ssl_tls_clienthello_parse_01);
 	tcase_add_test(tc, ssl_tls_clienthello_parse_02);
@@ -792,14 +855,36 @@ ssl_suite(void)
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ssl_is_ocspreq");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
 	tcase_add_test(tc, ssl_is_ocspreq_01);
 	suite_add_tcase(s, tc);
 
 	tc = tcase_create("ssl_features");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
 	tcase_add_test(tc, ssl_features_01);
 	tcase_add_test(tc, ssl_features_02);
 	suite_add_tcase(s, tc);
 
+	tc = tcase_create("ssl_key_refcount_inc");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
+	tcase_add_test(tc, ssl_key_refcount_inc_01);
+	suite_add_tcase(s, tc);
+
+	tc = tcase_create("ssl_x509_refcount_inc");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
+	tcase_add_test(tc, ssl_x509_refcount_inc_01);
+	suite_add_tcase(s, tc);
+
+#ifndef OPENSSL_NO_ENGINE
+	tc = tcase_create("ssl_engine");
+	tcase_add_checked_fixture(tc, ssl_setup, ssl_teardown);
+	tcase_add_test(tc, ssl_engine_01);
+	suite_add_tcase(s, tc);
+#else /* OPENSSL_NO_ENGINE */
+	fprintf(stderr, "ssl: 1 test omitted because OpenSSL has no "
+	                "engine support\n");
+#endif /* OPENSSL_NO_ENGINE */
+
 	return s;
 }
 
